/***************************************************************************/
/* 		This code is part of WWW graber called pavuk		   */
/*		Copyright (c) 1997,1998,1999 Ondrejicka Stefan		   */
/*		(ondrej@idata.sk)					   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#ifndef _config_h_
#define _config_h_

#define NeedFunctionPrototypes 1

#ifndef HAVE_LONG_FILE_NAMES
#warning "This program can't run successfuly on machine without long filenames support"
#endif

#include <time.h>
#include <fcntl.h>
#include <sys/types.h>
#include <netinet/in.h>

#include "mode.h"

#ifdef X_FACE
#include "xinterface.h"
#include "icons.h"
#else
#ifdef GTK_FACE
#include "ginterface.h"
#include "icons.h"
#else
#define _Xt_Serve
#define _Xt_ServeBr
#define _X_EscapeLoop
#define _Xt_ServeLoop
#endif
#endif

#ifndef O_BINARY
#define O_BINARY 0
#endif

#ifdef HAVE_FLOCK
#define _flock(fd) flock(fd , LOCK_EX | LOCK_NB)
#define _funlock(fd) flock(fd , LOCK_UN)
#else
#ifdef HAVE_FCNTL_FLOCK
/*
#define _flock(fd) fcntl(fd , F_SETLK , F_WRLCK)
#define _funlock(fd) fcntl(fd , F_SETLK , F_UNLCK)
*/
#define _flock(fd) my_flock(fd)
#define _funlock(fd) my_funlock(fd)
#else
#define _flock(fd) 0
#define _funlock(fd) 0
#endif
#endif

#include <limits.h>
#ifndef PATH_MAX
#define PATH_MAX	2048
#endif
#ifndef INT_MAX
#define INT_MAX		2147483647
#endif
#ifndef USHRT_MAX
#define	USHRT_MAX	65535
#endif

#ifdef SOCKS

#ifdef HAVE_SOCKS_H
#include <socks.h>
#endif

#ifndef SOCKS5
#define connect 	Rconnect
#define bind		Rbind
#define accept		Raccept
#define listen		Rlisten
#define select		Rselect
#define gethostbyname	Rgethostbyname
#define getsockname	Rgetsockname
#endif

#endif


#include "bufio.h"
#include "url.h"
#include "condition.h"
#include "mode.h"
#include "doc.h"
#include "errcode.h"
#include "nls.h"
#include "authinfo.h"
#include "cookie.h"

#ifdef USE_SSL
#include <stdio.h>
#include <ssl.h>
#endif

#define HASH_TBL_SIZE	233

typedef struct _htbl {
	int	nr;
	url	**urls;
} htbl;


typedef struct {
#ifdef DEBUG
	bool	debug;		/*** debug mode on/off ***/
#endif
#ifdef USE_SSL
	SSL	*ssl_con;
	SSL_CTX	*ssl_ctx;
	SSL_METHOD *ssl_method;
	char	*ssl_cipher_list;
	char	*ssl_cert_file;
	char	*ssl_key_file;
	char	*ssl_cert_passwd;
#endif
	bool	check_limits;
	char	**cookies_disabled_domains; /*** domains from which cookies are not acceptable ***/
	int	cookies_max;	/*** maximal number of cookies ***/
	int	cookies_num;	/*** total number of_cookies ***/
	cookie_entry *cookies;	/*** cookies data ***/
	cookie_entry *last_cookie; /*** last cookie entry ***/
	int	cookie_changed; /*** was mem cookies changed after last update on disk ***/
	time_t	cookie_updated;	/*** time when cookies were last updated on disc ***/
	char	*cookie_file;	/*** cookie file in NS format ***/
	bool	send_cookies;	/*** send available cookies in HTTP request ***/
	bool	recv_cookies;	/*** receive cookie infos ***/
	bool	update_cookies; /*** update cookie file ***/
	bool	preserve_time;	/*** preserve time of remote document ***/
	bool	quiet;		/*** ? output messages ***/
	char	*local_host;	/*** hostname of local machine ***/
	int	dns_pid;	/*** pid of DNS resolver process ***/
	bufio	*dns_w;		/*** write side of pipe to DNS resolver ***/
	bufio	*dns_r;		/*** read side of pipe to DNS resolver ***/
	int	base_level;	/*** base tree level from which is built local file tree ***/
	int	fail_cnt;	/*** counter for failed transfers ---> return code of pavuk ***/
	char	*auth_file;	/*** file where authentification informations are stored ***/
	authinfo **authdata;	/*** authentification informations ***/
	int	authdata_num;	/*** number of entries in authdata ***/
	char	*auth_realm;	/*** realm for HTTP digest access authentification ***/
	char	*auth_nonce;	/*** nonce for HTTP digest access authentification ***/
	char	*auth_opaque;	/*** opaque for HTTP digest access authentification ***/
	bool	auth_digest;	/*** is required digest authentification ? */
	bool	ftp_fatal_err;	/*** was the FTP error fatal ? ***/
	int	origsize;	/*** size of local copy of document ***/
	bool	show_time;	/*** show start and end time of download ? */
	bool	remove_old;	/*** remove old documents (when not occurrs on remote site) ***/
	bool	report_size;	/*** show progres internal ***/
	bool	progres;	/*** show retrieving progres , when on console ***/
	char	*save_scn;	/*** name for scenario saving ***/
	char    *scndir;	/*** directory where scenarios are stored ***/
	char	*scenario;	/*** scenario file to load ***/
	int	total_len;	/*** celkova dlzka aktualneho dokumentu ***/
	int	mode;		/*** mod behu programu ***/
	int	errcode;	/*** chybovy kod ***/
	char	*prg_path;	/*** nazov programu pod akym bol spusteny ***/
	char	*subdir;	/*** podadresar na ktory sa vztahuju akcie ***/
	char	*cache_dir;	/*** adresar do ktoreho sa uklada lokalny strom dokumentov ***/
	char	*http_proxy;	/*** HTTP proxy server ***/
	int	http_proxy_port;
	char	*http_proxy_pass; /*** password for access to HTTP proxy server ***/
	char	*http_proxy_user; /*** username for access to HTTP proxy server ***/
	int	proxy_auth_scheme; /*** Authorization scheme for proxy server ***/
	char	*ftp_proxy;	/*** FTP proxy server ***/
	int	ftp_proxy_port;
	bool	ftp_via_http;	/*** gatewaying FTP through HTTP proxy ***/
	bool	ftp_dirtyp;	/*** dirty FTP proxying via CONNECT request to HTTP proxy ***/
	bool	ftp_activec;	/*** use active FTP data connection instead of passive ***/
	bufio	*ftp_control;	/*** deskriptor soketu riadiaceho FTP spojenia ***/
	bufio	*ftp_datasock;	/*** active FTP data connection socket ***/
	int	ftp_respc;	/*** last ftp response ***/
	char	*ssl_proxy;	/*** SSL tuneling proxy ***/
	int	ssl_proxy_port;
	char	*gopher_proxy;	/*** Gopher proxy ***/
	int	gopher_proxy_port;
	bool	gopher_via_http; /*** gatewaying Gopher trough HTTP proxy ***/
	bool	cache;		/*** disallow caching of HTTP documents (on proxy cache) ***/
	url	**urlstack;	/*** zoznam vsetkych spracovavnych URL ***/
	char	**prepurlstr;	/*** zoznam nakonfigurovanych url ***/
	int	actual_url;	/*** index aktualne spracovavaneho URL ***/
	int	total_cnt;	/*** celkovy pocet URL v zozname ***/
	int	sync_cnt;	/*** number of URLs to synchronize from begining of urlstack ***/ 
	cond	condition;	/*** struktura v ktorej su obmedzujuce podmienky pre vyber dokumentov na spracovanie ***/
	int	ctimeout;	/*** timeout pri sietovej komunikacii ***/
	int	t_error;	/*** indikacia chyby pri prenose ***/
	struct tm *time;	/*** cas naplanovania spustenie procesu cez at ***/
	time_t	schtime;	/*** cmdln param for scheduling time ***/
	int	reschedh;	/*** number of hours for rescheduling ***/
	doc	*act_doc;	/*** aktualny dokument ***/
	bufio	*s_sock;		/*** deskriptor suboru do ktoreho sa online zapisuje ***/
	char *	lock_fn;	/*** name of lock file ***/
	bool	remove_lock;	/*** remove lock file if required ***/
	char	*name_auth;	/*** meno pouzivatela pri HTTP autentifikacii ***/
	char	*passwd_auth;	/*** heslo pre HTTP autentifikaciu ***/
	int	auth_scheme;	/*** autentifikacna schema 1- user , 2- Basic ***/
	char	*from;		/*** polozka From: alebo heslo pri anonymous FTP spojeni ***/
	bool	rewrite_links;	/*** indikacia ci ma maju byt odkazy v HTML dokumentoch prepisovane ***/
	int	nretry;		/*** pocet opakovani pokusov o spojenie alebo prenos dokumentu pri chybe ***/
	char	**accept_lang;	/*** zoznam preferovanych jazykov ***/
	int	sleep;		/*** cas v sekundach, pocas ktoreho bude program uspany ***/
	bool	use_enc;	/*** indikacia ci sa ma pouzivat gzip/compress kodovanie pri prenose ***/
	bool	is_sock;	/*** indikacia ci datove spojenie je lokalne alebo sietove ***/
	char	*buffer;	/*** adrresa buffera do ktoreho sa citaju udaje v ReadData***/
	int	totsz;		/*** celkova velkost ak je znama ***/
	int	rollback;	/*** size to go back when reget-ing ***/
	int	nreget;		/*** number of regets ***/
	int	nredir;		/*** max number of redirections ***/
	int	freget;		/*** force reget whole file when server doesn't support reget ***/
	int	ddays;		/*** delay in days for document reload ***/
	int	prev_mode;	/*** previous active mode ***/
	bool	mode_started;	/*** mode startup finisched ***/
	char 	*logfile;	/*** logging file ***/
	int	logfd;		/*** descriptor of log file ***/
#ifdef GETTEXT_NLS
	char	*msgcatd;	/*** explicit message catalog directory ***/
	char	*language;	/*** language for LC_MESSAGES ***/
#endif
#ifdef X_FACE
	Xnfo	X;
#endif

#ifdef GTK_FACE
	Gtk_nfo	gtk;
#endif

#ifdef I_FACE
	char	*fontname;	/*** default font used in interface ***/
	int	xlogsize;	/*** max number of lines in LOG widget ***/
#ifdef WITH_TREE
	char	*browser;	/*** cesta k browseru ktory sa spusti pri na dane URL ***/
	urltype_icon icon;	/*** icons in URL tree preview ***/
#endif
	bool	xi_face;	/*** indikacia ci je funkcne rozhranie X-windows ***/
	int	sock;		/*** aktualny datovy soket ***/
	bool	d_eof;		/*** bolo citanie ukoncane ? ***/
	bool	stop;		/*** pozaduje pouzivatel prerusenie po spracovani aktualneho dokumenu ***/
	bool	rbreak;		/*** pozaduje pouzivatel okamzite prerusenie prenosu ? ***/
	bool	done;		/*** was done startup ? ***/
	bool	processing;	/*** some URL is actualy in processing ***/
#endif

	htbl	url_hash_tbl[HASH_TBL_SIZE];	/*** hash table for better performance URL lookup ***/
} _config_struct;

extern _config_struct cfg;

typedef enum {
	PARAM_NUM ,
	PARAM_PBOOL ,
	PARAM_NBOOL ,
	PARAM_STR ,
	PARAM_STRLIST ,
	PARAM_CONN ,
	PARAM_AUTHSCH ,
	PARAM_MODE ,
	PARAM_PATH ,
	PARAM_TIME ,
	PARAM_HTMLTAG 
} par_type;

typedef struct _cfg_param {
	char		*par_cmd;
	char		*par_entry;
	char		*par_env;
	par_type	type;
	void		*default_val;
	void		*val_adr;
	void		*mdefault_val;
	void		*mval_adr;
	char		*help;
} cfg_param;

extern void report_error(int , char *);
extern void usage(char *);
extern void cfg_setup_default();
extern void cfg_setup_cmdln(int , char **);
extern int cfg_load(char *);
extern void cfg_load_setup();
extern int cfg_dump(char *);
extern int cfg_dump_cmd(char *);

#endif
