/***************************************************************************/
/* 		This code is part of WWW graber called pavuk		   */
/*		Copyright (c) 1997,1998,1999 Ondrejicka Stefan		   */
/*		(ondrej@idata.sk)					   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>

#include "config.h"
#include "decode.h"
#include "tools.h"

#define BUFSIZE	1024

/********************************************************/
/* zapise obsah pamete na adrese buf s dlzkou len	*/
/* na deskriptor fd					*/
/********************************************************/
static int writerm(len , buf , fd)
int len;
char *buf;
int fd;
{
	char *p = buf;
	int fsize = BUFSIZE;


	while (p < (buf + len))
	{
		if ((p + BUFSIZE) > (buf + len)) fsize = buf + len - p;
		if (write(fd , p , fsize) < 0)
		{
			return -1;
		}
		p += BUFSIZE;
	}

	close(fd);
	return 0;
}

/********************************************************/
/* vypis subor do deskriptora fd			*/
/********************************************************/
static int writerf(fname , fd)
char *fname;
int fd;
{
	char buf[BUFSIZE];
	int len;
	int fnfd;


	fnfd = open(fname , O_BINARY | O_RDONLY);
	if (fnfd < 0)
	{
		return -1;
	}

	while ((len = read(fnfd , buf, sizeof(buf))) > 0 || errno == EWOULDBLOCK)
	{
		if (write(fd , buf ,len) != len)
		{
			close(fd);
			close(fnfd);
			return -1;
		}
	}

	close(fd);
	close(fnfd);
	return 0;
}

/********************************************************/
/* dekompresia pomocou gzipu				*/
/* pamet na adrese inbuf s dlzkou insize cez ruru posle */
/* gzipu a zo standartneho vystupu gzipu precita 	*/
/* dekompimovane data					*/
/********************************************************/
int gzip_decode(inbuf , insize , outbuf , outsize , fname)
char *inbuf;
int insize;
char **outbuf;
int *outsize;
char *fname;
{
	int writepipe[2],readpipe[2];
	pid_t pidg,pidw;
	char *out = NULL;
	int outcnt = 0,len;
	char buf[BUFSIZE];
	int status1,status2;

	if (pipe(writepipe))
	{
		xperror("pipe");
		return -1;
	}
	if (pipe(readpipe))
	{
		xperror("pipe");
		close(writepipe[0]);
		close(writepipe[1]);
		return -1;
	}

	pidg = fork();
	
	if (pidg < 0)
	{
		xperror("fork");
		close(writepipe[0]);
		close(writepipe[1]);
		close(readpipe[0]);
		close(readpipe[1]);
		return -1;		
	}
	if (pidg == 0)
	{
		close(0);
		dup(writepipe[0]);
		close(1);
		dup(readpipe[1]);
		close(writepipe[0]);
		close(writepipe[1]);
		close(readpipe[0]);
		close(readpipe[1]);
		
		execlp(GZIP_CMD , "gunzip" , "-cf" , NULL);
		xperror("gunzip exec");
		exit(1);
	}
	
	close(readpipe[1]);
	close(writepipe[0]);
	
	pidw = fork();
	
	if (pidw < 0)
	{
		xperror("fork");
		close(writepipe[0]);
		close(writepipe[1]);
		close(readpipe[0]);
		close(readpipe[1]);
		waitpid(pidg , &status2 , 0);
		return -1;		
	}

	if (pidw == 0)
	{
		close(readpipe[0]);
		if (fname) exit(writerf(fname , writepipe[1]));
		else exit(writerm(insize , inbuf , writepipe[1]));	
	}

	close(writepipe[1]);

	while((len = read(readpipe[0] , buf , sizeof(buf))) > 0)
	{
		out = (char *) _realloc(out , outcnt + len + 1);
		memcpy(out + outcnt , buf , len);
		outcnt += len;
		*(out + outcnt) = '\0';
	}

	close(readpipe[0]);

	waitpid(pidw , &status1 , 0);
	waitpid(pidg , &status2 , 0);

	if (len < 0 || status1 || status2)
	{
		_free(outbuf);
		return -1;
	}
	*outsize = outcnt;
	*outbuf = out;
	return 0;
}
