/***************************************************************************/
/* 		This code is part of WWW graber called pavuk		   */
/*		Copyright (c) 1997,1998,1999 Ondrejicka Stefan		   */
/*		(ondrej@idata.sk)					   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <utime.h>

#include "config.h"
#include "tools.h"
#include "url.h"
#include "doc.h"
#include "condition.h"
#include "html.h"


#define NARAZNIK	4096
#define MEXPAND		if ((sz - r) < NARAZNIK)\
			{\
				 result = _realloc(result , sz + NARAZNIK);\
				 sz += NARAZNIK;\
			}

/************************************************************/
/* tabulka znamych znaciek a atributov ktore sa spracovavaju*/
/************************************************************/
html_tag linky[23] =
{
	{"BODY" , 
		{{"BACKGROUND" , LINK_INLINE |  LINK_DOWNLD} , 
		 {NULL , 0}}},
	{"IMG" , 
		{{"SRC" , LINK_INLINE | LINK_DOWNLD},
		 {"LONGDESC" , LINK_DOWNLD},
		 {"USEMAP" , 0},
		 {NULL , 0}}},
	{"INPUT" ,
		{{"SRC" , LINK_INLINE | LINK_DOWNLD},
		 {"USEMAP" , 0},
		 {NULL , 0}}},
	{"FRAME" , 
		{{"SRC" , LINK_INLINE | LINK_DOWNLD},
		 {"LONGDESC" , LINK_DOWNLD},
		 {NULL , 0}}},
	{"IFRAME" , 
		{{"SRC" , LINK_DOWNLD},
		 {"LONGDESC" , LINK_DOWNLD},
		 {NULL , 0}}},
	{"APPLET" , 
		{{"CODEBASE" , 0},
		 {NULL , 0}}},
	{"SCRIPT" , 
		{{"SRC" , 0},
		 {NULL , 0}}},
	{"BGSOUND" , 
		{{"SRC" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"EMBED" , 
		{{"SRC" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"AREA" , 
		{{"HREF" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"BASE" , 
		{{"HREF" , 0},
		 {NULL , LINK_REMOVE}}},
	{"FIG" , 
		{{"SRC" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"OVERLAY" , 
		{{"SRC" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"A" , 
		{{"HREF", LINK_DOWNLD},
		 {NULL , 0}}},
	{"LINK" , 
		{{"HREF" , LINK_DOWNLD},
		 {NULL , 0}}},
	{"FORM" , 
		{{"ACTION" , 0},
		 {NULL , 0}}},
	{"LAYER" , 
		{{"SRC" , LINK_DOWNLD} ,
		 {NULL , 0}}},
	{"META" , 
		{{"URL" , LINK_DOWNLD},
		 {NULL , 0}}},
	{"INS" , 
		{{"CITE" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"DEL" , 
		{{"CITE" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"Q" , 
		{{"CITE" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"SPAN" , 
		{{"HREF" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
	{"DIV" , 
		{{"HREF" , LINK_INLINE | LINK_DOWNLD},
		 {NULL , 0}}},
};

/**** makro zistujuce ci znak je LWS ****/
#define IS_SEP(p) (((p) == '\n') || ((p) == ' ') || ((p) == '\t') || ((p) == '\r'))

/**** deklaracie statickych funkcii ****/
#if NeedFunctionPrototypes
static void replace_url_in_stack(char *,char * , char * , int);
static int html_to_absolute_links(char **,doc *);
#else
static void replace_url_in_stack();
static int html_to_absolute_links();
#endif

/*******************************************************/
/* zo znacky vytiahne hodnodu specifikovaneho atributu */
/*******************************************************/
char *get_url_from_tag(tag,link_attrib)
char *tag;
char *link_attrib;
{
	char *pom = new_string(tag);
	char *p;
	char *retval = NULL;

	p = strtok(pom , "= \t");	

	while(p && strncasecmp(p , link_attrib , strlen(link_attrib))) 
		p = strtok(NULL , "= \t");

	if (p)
		{
		p = strtok(NULL , "");	
		if (p)
		{
			while(IS_SEP(*p) || *p == '=') p++;
			if ((*p == '\"'))
			{
				p = strtokc(p + 1,'\"');
			}
			else
			{
				p = strtok(p," \t\"");
			}
			while(IS_SEP(*p)) p++;
			retval = new_string(p);
		}
	}
	free(pom);
	return retval;	
}

/*** look if tag contains specified element ***/
static int tag_co_elem(tag , elem)
char *tag;
char *elem;
{
	char *pom = new_string(tag);
	char *p;
	int ret_val;

	p = strtok(pom , "= \t");	

	while(p && strcasecmp(p , elem))
		p = strtok(NULL , "= \t");

	ret_val = (p == NULL) ? 0 : !strcasecmp(p ,elem);
	free(pom);

	return ret_val;
}

/****************************************************************/
/* V HTML dokumente najde vsetky podporovane URL splnajuce	*/
/* obmedzujuce podmienky					*/
/****************************************************************/ 
url **html_get_all_links(html_doc , only_inline)
doc *html_doc;
int only_inline;
{
	char stack[8192];
	char *sp = stack;
	char *p,*idx;
	char *psp;
	bool imgstart = FALSE;
	url **retval = (url **)_malloc(sizeof(url *));
	int numimg = 0;
	url *purl;
	int i = 0 , j = 0;
	int prev_tag = NUM_ELEM(linky);
	int old_tag=0;
	char *base,*baset;

	/**** nastavenie zakladneho URL dokumentu ****/

	baset = url_to_urlstr(html_doc->doc_url);
	if ((p = strrchr(baset, '#')))
	{
		*p = '\0';
	}

	base = url_to_urlstr(html_doc->doc_url);

	if (*(base + strlen(base)) != '/')
	{
		p = strrchr(base , '/');
		if (p) *(p + 1) = '\0';
	}

	retval[0] = NULL;

	/**** prechazame cely dokument po znakoch ****/
	for(p = html_doc->contents ; (p - html_doc->contents) < html_doc->size ; p++)
	{
		/**** vyhodime nepotrebne separatory ****/
		if (isspace(*p))
		{
			if (imgstart)
			{
				*sp = ' ';
				sp++;
			}
			continue;
		}

		/**** zacina znacka ****/
		if (*p == '<')
		{
			p++;
			imgstart = FALSE;
			old_tag = prev_tag;

			/**** zisti ci je to niektora znacka z tabulky ****/
			for (i = 0 ; i < NUM_ELEM(linky) ; i++)
			{
				if (!strncasecmp(p , linky[i].tag , strlen(linky[i].tag)) 
					&& IS_SEP(*(p + strlen(linky[i].tag))))
				{
					p += strlen(linky[i].tag) - 1;
					sp = stack;
					imgstart = TRUE;
					break;
				}
			}
			prev_tag = i;

			sp = stack;
			if (imgstart) continue;
			p--;
		}
		/**** je koniec znacky ****/
		if (*p == '>' && imgstart)
		{
			/* zarad image */
			imgstart = FALSE;
			*sp = '\0';
			/**** vyber URL zo znacky ****/
			/* !!! inline !!! */
			for(j = 0 ; linky[i].attribs[j].attrib ; j++)
			{
				if (linky[i].attribs[j].stat & LINK_DISABLED)
					continue;

				psp = get_url_from_tag(stack,linky[i].attribs[j].attrib);

				if (psp)
				{
					/**** nove zakladne URL ****/
					if (!strcasecmp(linky[i].tag,"BASE") && 
					    !strcasecmp(linky[i].attribs[j].attrib,"HREF") && psp)
					{
						purl = parse_url(psp);
						if ((purl->type == FILET) && 
							(html_doc->doc_url->type == HTTP ||
							 html_doc->doc_url->type == HTTPS ||
							 html_doc->doc_url->type == FTP))
						{
							char pom[2048];

							if (*(purl->p.file.filename) == '/')
							{
								strcpy(pom , base);
								idx = strfindnchr(pom , '/' , 3);
								if (idx)
									strcpy(idx + 1 , 
										purl->p.file.filename );
								else
								{
									strcat(pom , "/");
									strcat(pom , 
										purl->p.file.filename );
								}
							}
							else
							{
				
								sprintf(pom , "%s%s" , base ,
									purl->p.file.filename);
							}
							free_deep_url(purl);
							free(purl);
							purl = parse_url(pom);
						}
						free(base);
						base = url_to_urlstr(purl);
						if (*(base + strlen(base)) != '/')
						{
							idx = strrchr(base , '/');
							if (idx) *idx = '\0';
						}
						free_deep_url(purl);
						free(purl);
					}
					else if ((linky[i].attribs[j].stat & LINK_DOWNLD) &&
						 (!only_inline || (only_inline && linky[i].attribs[j].stat & LINK_INLINE)))
					{
						char *pom = url_to_absolute_url(base , baset , 
							html_doc->doc_url , psp);

						if (pom)
						{
							_free(psp);
							psp = pom;
						}

						purl = parse_url(psp);
						purl->parent_url = (url **)_malloc(2 * sizeof(url *));
						purl->parent_url[0] = html_doc->doc_url;
						purl->parent_url[1] = NULL;
						purl->level = html_doc->doc_url->level + 1;
						if (linky[i].attribs[j].stat & LINK_INLINE) 
							purl->status |= URL_INLINE_OBJ;
						url_path_abs(purl);


						if ((cfg.mode == MODE_SYNC) && cfg.prepurlstr && (purl->type == FILET))
						{
							url *pomurl = filename_to_url(purl->p.file.filename);
							if (pomurl)
							{
								free_deep_url(purl);
								_free(purl);
								purl = pomurl;
							}
						}

						/**** previous URL refer to server side map 
						      (not handled) ****/
						if (old_tag < NUM_ELEM(linky) &&
						    !strcasecmp(linky[i].tag,"IMG") &&
						    !strcasecmp(linky[old_tag].tag,"A") &&
						    tag_co_elem(stack , "ISMAP"))
						{
							free_deep_url(retval[numimg - 1]);
							_free(retval[numimg - 1]);
							numimg --;
						}
				
						/**** splna URL obmedzujuce podmienky ****/
						if (url_append_condition(purl,TRUE))
						{
							/**** pridanie do zoznamu ****/
							numimg ++;
							retval = _realloc(retval , 
								(numimg + 1) * sizeof(url *));
							retval[numimg - 1] = purl;
							retval[numimg] = NULL;
						}
						else
						{
							prev_tag = NUM_ELEM(linky);
							free_deep_url(purl);
							free(purl);
						}
					}
					_free(psp);
				}
			}
			imgstart = FALSE;
			continue;
		}
		/**** vkladanie znakov na zasobnik ak bol zaciatok podporovanej znacky ****/
		if (imgstart)
		{
			*sp = *p;
			sp++;
		}
	}
	_Xt_Serve;

	free(base);
	free(baset);
	return retval;
}

/************************************************/
/* spojenie dvoch zoznamov URL do jedneho 	*/
/************************************************/

void cat_links_to_url_list(l1)
url **l1;
{
	url **p = l1;
	url *same;

	while (p && *p)
	{
		if (url_append_condition(*p , TRUE))
		{
			if ((same = url_was_befor(*p)))
			{
				link_url_in_list(same , *p);
				free_deep_url(*p);
				_free(*p);
			}
			else
				append_url_to_list(*p);
		}
		else
		{
			free_deep_url(*p);
			_free(*p);
		}

		p++;
	}

	_free(l1);
}


/************************************************/
/* prepisanie hodnoty atributu znacky na mieste	*/
/*************************************i**********/
static void replace_url_in_stack(tag , link_attrib , urlin , pare)
char *tag;
char *link_attrib;
char *urlin;
int pare;
{
	char *pom = new_string(tag);
	char *poms = new_string(tag);
	char *p,*urls,*urle,*p2;
	char waste[1024];
	int i = 0;

	waste[0] = '\0';

	p = strtok(pom , "= \t");

	while(p && strncasecmp(p , link_attrib , strlen(link_attrib))) 
		p = strtok(NULL , "= \t");

	if (p)
	{
		p = strtok(NULL , "");	
		urls = p;
		if (p)
		{
			while(IS_SEP(*p)  || *p == '=')
			{
				waste[i] = *p;
				waste[i+1] = '\0';
				i++;
				p++;
			}

			if ((*p == '\"'))
			{
				p = strtokc(p + 1,'\"');
				p2 = strtokc(NULL , '\0');
			}
			else
			{
				p = strtok(p," \t");
				p2 = strtok(NULL , "");
			}

			urle = p + strlen(p) + 1;
			*(poms + (urls - pom)) = '\0';

			if (p2 && *p2)
			{
				if (!pare)
					sprintf(tag , "%s%s\"%s\" %s>" , poms , waste , urlin , p2);
				else
					sprintf(tag , "%s%s%s\" %s>" , poms , waste , urlin , p2);
			}
			else
			{
				if (!pare)
					sprintf(tag , "%s%s\"%s\">" , poms , waste , urlin);
				else
					sprintf(tag , "%s%s%s\">" , poms , waste , urlin);
					
			}
		}
	}

	free(pom);
	free(poms);
}

/****************************************************************/
/* prepisanie relativnych URL v HTML dokumente na absolutne 	*/
/****************************************************************/
static int html_to_absolute_links(content , html_doc)
char **content;
doc *html_doc;
{
	char stack[8192];
	char *sp = stack;
	char *p;
	bool tagstart = FALSE;
	char *result;
	int sz,r;
	url *purl;
	char *psp;
	int i = 0 , j = 0;
	char *base,*idx,*baset;


	sz = html_doc->size + NARAZNIK;
	result = _malloc(sz);
	r = 0;

	baset = url_to_urlstr(html_doc->doc_url);
	if ((p = strrchr(baset, '#')))
	{
		*p = '\0';
	}

	base = url_to_urlstr(html_doc->doc_url);
	if (*(base + strlen(base)) != '/')
	{
		p = strrchr(base , '/');
		if (p) *(p + 1) = '\0';
	}

	for (p = html_doc->contents  ; (p - html_doc->contents) < html_doc->size ; p++)
	{
		if (isspace(*p))
		{
			if (!tagstart)
			{
				result[r] = *p;
				r++;
				result[r] = '\0';
				MEXPAND;
			}
			else
			{
				*sp = ' ';
				sp++;				
			}
			continue;
		}
		if (*p == '<')
		{
			p++;

			tagstart = FALSE;

			for (i = 0 ; i < NUM_ELEM(linky) ; i++)
			{
				if (!strncasecmp(p , linky[i].tag , strlen(linky[i].tag))
				 	&& IS_SEP(*(p + strlen(linky[i].tag))))
				{
					strcpy(result + r , "<");
					strncpy(result + r + 1 , p , strlen(linky[i].tag));
					r += strlen(linky[i].tag) + 1;
					result[r] = '\0';
					MEXPAND;
					p += strlen(linky[i].tag) - 1;
					sp = stack;
					tagstart = TRUE;

					break;
				}
			}
			if (tagstart) continue;
			p--;
		}
		if (*p == '>' && tagstart)
		{
			*sp = '\0';
			for(j = 0 ; linky[i].attribs[j].attrib ; j++)
			{
				psp = get_url_from_tag(stack,linky[i].attribs[j].attrib);
				if (psp)
				{
					if (!strcasecmp(linky[i].tag,"BASE"))
					{
						purl = parse_url(psp);

						if ((purl->type == FILET) && 
							(html_doc->doc_url->type == HTTP ||
							 html_doc->doc_url->type == HTTPS ||
							 html_doc->doc_url->type == FTP))
						{
							char pom[2048];
	
							if (*(purl->p.file.filename) == '/')
							{
								strcpy(pom , base);
								idx = strfindnchr(pom , '/' , 3);
								if (idx)
									strcpy(idx , 
										purl->p.file.filename );
								else
								{
									strcat(pom , "/");
									strcat(pom , 
										purl->p.file.filename );
								}
							}
							else
							{
								sprintf(pom , "%s%s" , base ,
									purl->p.file.filename);
							}

							free_deep_url(purl);
							free(purl);
							purl = parse_url(pom);
						}
						free(base);
						base = url_to_urlstr(purl);
						if (*(base + strlen(base)) != '/')
						{
							idx = strrchr(base , '/');
							if (idx) *p = '\0';
						}
						free(psp);
						free_deep_url(purl);
						free(purl);
						r -= 5;
						tagstart = FALSE;
						continue;
					} 
					else
					{
						char *pom = url_to_absolute_url(base , baset , 
							html_doc->doc_url , psp);
					
						if (pom)
						{
							free(psp);
							purl = parse_url(pom);
							_free(pom);
							url_path_abs(purl);
							psp = url_to_urlstr(purl);
							free_deep_url(purl);
							_free(purl);
						}

						replace_url_in_stack(stack ,
							linky[i].attribs[j].attrib , psp ,
							(!strcasecmp(linky[i].tag,"META") && 
							 !strcasecmp(linky[i].attribs[j].attrib,"URL")));

						_free(psp);
					}
				}
			}
				
			strcpy(result + r , stack);
			r += strlen(stack);
			if (result[r - 1] != '>') {result[r] = '>'; r++;}
			MEXPAND;
			tagstart = FALSE;
			continue;
		}
		if (tagstart)
		{
			*sp = *p;
			sp++;
		}else
		{
			result[r] = *p;
			r++;
			result[r] = '\0';

			MEXPAND;
		}
	}
	free(base);
	free(baset);
	*content = result;
	return (r);
}

/************************************************/
/* prepisanie relativnych  URL dokumentu 	*/
/************************************************/
void relocate_html_doc(docu)
doc *docu;
{
	int s;
	char *p;

	if (docu->doc_url->type != HTTP && 
	    docu->doc_url->type != HTTPS && 
	    docu->doc_url->type != FTP) return;

	if ((docu->doc_url->status & URL_REDIRECT)) return;

	s = html_to_absolute_links(&p , docu);

	free(docu->contents);
	docu->contents = p;
	docu->size = s;

}


/****************************************************************/
/* prepisanie vzdialenych URL v HTML dokumente na lokalne	*/
/****************************************************************/
int html_remote_to_local_links(contentout , html_doc)
char **contentout;
doc *html_doc;
{
	char stack[8192];
	char pom[4096];
	char *sp = stack;
	char *p;
	int sz,r;
	bool tagstart = FALSE;
	char *result;
	char *psp;
	int i=0,j=0,link_idx=0;
	char *act_name = url_to_filename(html_doc->doc_url);

	sz = NARAZNIK + html_doc->size;
	r = 0;
        result = _malloc(sz);

	for (p = html_doc->contents  ; (p - html_doc->contents) < html_doc->size ; p++)
	{
		if (isspace(*p))
		{
			if (!tagstart)
			{
				result[r] = *p;
				r++;
				result[r] = '\0';

				MEXPAND;
			}
			else
			{
				*sp = ' ';
				sp++;
			}
			continue;
		}
		if (*p == '<')
		{
			p++;
			tagstart = FALSE;

			for (i = 0 ; i < NUM_ELEM(linky) ; i++)
			{
				if (!strncasecmp(p , linky[i].tag , strlen(linky[i].tag))
				 	&& IS_SEP(*(p + strlen(linky[i].tag))))
				{
					strcpy(result + r , "<");
					strncpy(result + r + 1 , p , strlen(linky[i].tag));
					r += strlen(linky[i].tag) + 1;
					result[r] = '\0';
					MEXPAND;
					p += strlen(linky[i].tag) - 1;
					sp = stack;
					tagstart = TRUE;
					link_idx = i;
					break;
				}
			}
			if (tagstart) continue;

			p--;
		}
		if (*p == '>' && tagstart)
		{
			*sp = '\0';

			for(j = 0 ; linky[link_idx].attribs[j].attrib ; j++)
			{
				psp = get_url_from_tag(stack,linky[link_idx].attribs[j].attrib);

				if (psp)
				{
					url *urlp = parse_url(psp);
					char *fn = url_to_filename(urlp);
					struct stat estat;


					if (fn && ((!access(fn , R_OK) && !stat(fn , &estat)) || !strcmp(act_name , fn)))
					{
						if ((!access(fn , R_OK) && !S_ISDIR(estat.st_mode)) || !strcmp(act_name , fn))
						{
							char *relfn = get_relative_path(act_name , fn);

							if ((urlp->type == HTTP || urlp->type == HTTPS) 
								&& urlp->p.http.anchor_name)
								sprintf(pom,"%s#%s", relfn , urlp->p.http.anchor_name);
							else if (urlp->type == FILET && urlp->p.file.anchor_name)
								sprintf(pom,"%s#%s" , relfn , urlp->p.file.anchor_name);	
							else strcpy(pom,relfn);

							free(relfn);

						}
						else
	 					{
	 						strcpy(pom , fn);
	 						strcat(pom , "/_._.html");

		 					if (!stat(pom , &estat) &&
		 						!S_ISDIR(estat.st_mode))
	 						{
								char *relfn = get_relative_path(act_name , pom);
	
								if ((urlp->type == HTTP || urlp->type == HTTPS) && 
									urlp->p.http.anchor_name)
									sprintf(pom,"%s#%s", relfn , urlp->p.http.anchor_name);
								else if (urlp->type == FILET && urlp->p.file.anchor_name)
									sprintf(pom,"%s#%s" , relfn , urlp->p.file.anchor_name);	
								else strcpy(pom,relfn);

								free(relfn);
		 					}
		 				}
						replace_url_in_stack(stack , 
							linky[link_idx].attribs[j].attrib , pom ,
							(!strcasecmp(linky[i].tag,"META") && 
						         !strcasecmp(linky[i].attribs[j].attrib,"URL")));
					}
					free_deep_url(urlp);
					_free(urlp);
					_free(fn);
					_free(psp);
				}
			}
			strcat(result + r ,  stack);
                        r += strlen(stack);
			if (result[r - 1] != '>') {result[r] = '>'; r++;}
			MEXPAND;
			tagstart = FALSE;
			continue;
		}
		if (tagstart)
		{
			*sp = *p;
			sp++;
		}else
		{
			result[r] = *p;
			r++;
			result[r] = '\0';
			MEXPAND;
		}
	}
	_free(act_name);
	*contentout = result;
	return (r);
}



/********************************************************/
/* prepisanie specifickeho URL v HTML dokumente na ine	*/
/********************************************************/
static int html_change_url(contentout , html_doc , url_old , url_new)
char **contentout;
doc *html_doc;
url *url_old;
char *url_new;
{
	char stack[4096];
	char *sp = stack;
	char *oldfn,*pfn;
	char *p;
	int sz,r,i,j;
	int link_idx = 0;
	bool tagstart = FALSE;
	char *result;
	char *psp;
	url *purl;

	sz = NARAZNIK + html_doc->size;
	r = 0;
        result = _malloc(sz);

	oldfn = url_to_filename(url_old);

	for (p = html_doc->contents  ; (p - html_doc->contents) < html_doc->size ; p++)
	{
		if (isspace(*p))
		{
			if (!tagstart)
			{
				result[r] = *p;
				r++;
				result[r] = '\0';

				MEXPAND;
			}
			else
			{
				*sp = ' ';
				sp++;
			}
			continue;
		}
		if (*p == '<')
		{
			p++;
			tagstart = FALSE;

			for (i = 0 ; i < NUM_ELEM(linky) ; i++)
			{
				if (!strncasecmp(p , linky[i].tag , strlen(linky[i].tag))
				 	&& IS_SEP(*(p + strlen(linky[i].tag))))
				{
					strcpy(result + r , "<");
					strncpy(result + r + 1 , p , strlen(linky[i].tag));
					r += strlen(linky[i].tag) + 1;
					result[r] = '\0';
					MEXPAND;
					p += strlen(linky[i].tag) - 1;
					sp = stack;
					tagstart = TRUE;
					link_idx = i;
					break;
				}
			}
			if (tagstart) continue;

			p--;
		}

		if (*p == '>' && tagstart)
		{
			*sp = '\0';

			for(j = 0 ; linky[link_idx].attribs[j].attrib ; j++)
			{
				psp = get_url_from_tag(stack,linky[link_idx].attribs[j].attrib);
				if (psp)
				{
					purl = parse_url(psp);
					if ((pfn = url_to_filename(purl)))
					{
						if (!strcmp(pfn , oldfn))
						{
							char pom[4096];
							char *ppom;

							if ((purl->type == HTTP || purl->type == HTTPS) && purl->p.http.anchor_name)
							{
								sprintf(pom , "%s#%s" , url_new , purl->p.http.anchor_name);
								ppom = pom;
							}
							else
								ppom = url_new;
								
							replace_url_in_stack(stack , 
								linky[link_idx].attribs[j].attrib , url_new ,
								(!strcasecmp(linky[link_idx].tag,"META") && 
							         !strcasecmp(linky[link_idx].attribs[j].attrib,"URL")));
						}
					}
					free_deep_url(purl);
					free(purl);
					free(psp);
					_free(pfn);
				}
			}
                        strcat(result + r ,  stack );
                        r += strlen(stack);
			if (result[r - 1] != '>') {result[r] = '>'; r++;}
                        MEXPAND;
			tagstart = FALSE;
			continue;
		}
		if (tagstart)
		{
			*sp = *p;
			sp++;
		}else
		{
			result[r] = *p;
			r++;
			result[r] = '\0';
			MEXPAND;
		}
	}
	*contentout = result;
	free(oldfn);
	return (r);
}


/****************************************************************/
/* nacitanie dokumentu z ktoreho bol URL referencovane ,	*/
/* prepisanie vzdialeneho odkazu na relativny lokalny a spatne	*/
/* ulozenie							*/
/****************************************************************/
void rewrite_one_parent_links(doc_url , parent_url , dst_name)
url *doc_url;
url *parent_url;
char *dst_name;
{
	char pom[2048];
	char *fnamep;
	char *rfn;
	char *savetmp,*p;
	int sock,len;
	doc pdoc;
	char *newcont;
        struct stat estat;
	struct utimbuf ut;
        url dum;

	if (!parent_url || !(parent_url->status & URL_DOWNLOADED)) return;

	fnamep = url_to_filename(parent_url);
        if (stat(fnamep , &estat) == 0)
        {
                if (S_ISDIR(estat.st_mode))
                {
                        xprintf(1 , gettext("Can't work on directory\n"));
                        free(fnamep);
			return;
                }
        }
	else
	{
		xperror("stat");
		return;
	}

	ut.actime = estat.st_atime;
	ut.modtime = estat.st_mtime;

	dum.type = FILET;
	dum.p.file.filename = fnamep;
	dum.status &= ~URL_REDIRECT;
	pdoc.doc_url = &dum;
	pdoc.save_online = FALSE;
	pdoc.dtime = NULL;

	cfg.act_doc = &pdoc;
	cfg.is_sock = FALSE;

	cfg.report_size = FALSE;
	if (download_document(&pdoc , 1))
	{
		remove_lock();
		if (cfg.errcode) report_error(cfg.errcode , "rewrite parent");
		free(fnamep);
		return;
	}
	cfg.report_size = TRUE;

	if (cfg.errcode) report_error(cfg.errcode , "rewrite parent");

	_free(pdoc.mime);

	if (dst_name)
	{
		if  (!access(dst_name , R_OK) && !stat(dst_name , &estat))
		{
			if (!S_ISDIR(estat.st_mode))
			{
				rfn = get_relative_path(fnamep , dst_name);

				if ((doc_url->type == HTTP || doc_url->type == HTTPS) && 
					doc_url->p.http.anchor_name)
					sprintf(pom,"%s#%s", rfn , doc_url->p.http.anchor_name);
				else if (doc_url->type == FILET && doc_url->p.file.anchor_name)
					sprintf(pom,"%s#%s" , rfn , doc_url->p.file.anchor_name);	
				else strcpy(pom,rfn);

				free(rfn);

				len = html_change_url(&newcont , &pdoc , doc_url , pom);

				free(pdoc.contents);

				pdoc.contents = newcont;
				pdoc.size = len;
			}
		}
	}

	len = html_remote_to_local_links(&newcont , &pdoc);

	strcpy(pom , fnamep);
	p = strrchr(pom , '/');
	if (p)
		sprintf(p + 1 , "_*%d" , getpid());
	else
		sprintf(pom , "%s/_*%d" ,pom , getpid());

	savetmp = new_string(pom);	
	rename(fnamep , savetmp);

	if ((sock = open(fnamep , O_BINARY | O_CREAT | O_TRUNC | O_WRONLY , S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH)) < 0)
	{
		xperror(fnamep);
		rename(savetmp , fnamep);
		remove_lock();
		free(savetmp);
		free(fnamep);
		free(newcont);
		free(pdoc.contents);
		return;
	}
	if (write(sock , newcont , len) != len)
	{
		xperror(fnamep);
		close(sock);
		rename(savetmp , fnamep);
		remove_lock();
		free(savetmp);
		free(fnamep);
		free(newcont);
		free(pdoc.contents);
		return;
	}
	utime(fnamep , &ut);
	close(sock);
	remove_lock();
	unlink(savetmp);
	free(savetmp);
	free(fnamep);
	free(newcont);
	free(pdoc.contents);
}

void rewrite_parents_links(doc_url , dst_name)
url *doc_url;
char *dst_name;
{
	int i;
	char *fn = NULL;

	if ((doc_url->status & URL_MOVED) && !dst_name) return;
	
	for (i = 0; i < doc_url->ref_cnt ; i++)
	{
		if (!doc_url->parent_url[i]) continue;

		if (doc_url->parent_url[i]->status & URL_MOVED)
		{
			fn = dst_name ? dst_name : url_to_filename(doc_url);

			rewrite_parents_links(doc_url->parent_url[i] , fn);

			if (!dst_name && fn) _free(fn);
		}
		else
		{
			rewrite_one_parent_links(doc_url , doc_url->parent_url[i] , dst_name);
		}
	}
}
